/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
/* Tests for scrypt key derivation function.                                   (c) C.Veness 2018  */
/*                                                                                   MIT Licence  */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

import { expect } from 'chai'; // BDD/TDD assertion library

import Scrypt from '../scrypt.js';

const password = 'my secret password';


describe('Scrypt tests using ESM import', function() {

    describe('Hash & verify', function() {
        it('with just logN param', async function() {
            const key = await Scrypt.kdf(password, { logN: 12 });
            expect(Scrypt.viewParams(key)).to.deep.equal({ logN: 12, r: 8, p: 1 });
            expect(await Scrypt.verify(key, password)).to.be.true;
        });
    });

    describe('Error checking', function() {

        describe('kdf errors', function() {
            it('throws on numeric passphrase', () => Scrypt.kdf(99)
                .then(() => { throw new Error('Test should fail'); })
                .catch(error => expect(error.message).to.equal('Passphrase must be a string')));
        });

        describe('verify errors', function() {
            it('throws on bad key type', async () => Scrypt.verify(await Scrypt.kdf(password, { logN: 12 }), null)
                .catch(error => expect(error.message).to.equal('Passphrase must be a string')));
            it('throws on bad key type', () => Scrypt.verify(null, 'passwd')
                .then(() => { throw new Error('Test should fail'); })
                .catch(error => expect(error.message).to.equal('Key must be a string')));
            it('throws on bad key', () => Scrypt.verify('key', 'passwd')
                .then(() => { throw new Error('Test should fail'); })
                .catch(error => expect(error.message).to.equal('Invalid key')));
            it('throws on checksum failure', async () => {
                const keyBuff = Buffer.from(await Scrypt.kdf(password, { logN: 12 }), 'base64');
                keyBuff[7] = 11; // patch logN to new value
                const keybis = keyBuff.toString('base64');
                expect(Scrypt.viewParams(keybis)).to.deep.equal({ logN: 11, r: 8, p: 1 });
                expect(await Scrypt.verify(keybis, password)).to.be.false;
            });
        });

        describe('viewParams errors', function() {
            it('throws on null key', () => expect(() => Scrypt.viewParams(null)).to.throw(TypeError, 'Key must be a string'));
            it('throws on numeric key', () => expect(() => Scrypt.viewParams(99).to.throw(TypeError, 'Key must be a string')));
            it('throws on invalid key', () => expect(() => Scrypt.viewParams('bad key').to.throw(RangeError, 'Invalid key')));
        });

    });

});
